'###############################################################################
'# SCRIPT: RVGL updater
'# 
'# AUTHOR: m15t3r_gr1mm
'# 
'# DESCRIPTION:
'#  This script acts as the main entry point for rvgl.
'#  It should be treated as a replacement to rvgl.exe, and is fully capable of
'#  forwarding arguments on to the main binary in the same manner as before.
'#  To do this, create a shortcut to this script and modify the target path.
'#  This script adds the capability of checking for any updates to rvgl.exe before
'#  launching by comparing the local changelog against:
'#  https://distribute.re-volt.io/releases/rvgl_version.txt.
'# 
'# NOTES:
'#  This updater must be kept within the main install directory with rvgl.exe.
'#  If you wish to bypass checking for updates, use the new flag option: -noupdate
'#  in the target path of the shortcut (mentioned above).
'#  Also, if you prefer to have the RVGL updater pinned to the taskbar, the
'#  target path value MUST begin with: "wscript.exe", followed by the path to
'#  this script, and any parameters desired. This is a Windows OS bug - without
'#  wscript.exe specified, the updater will not be pinable and instead the
'#  Windows vbs script interpreter is pinned in its' place.
'#
'# REV HISTORY:
'#
'#  date         name            description
'# -----------  --------------  ---------------------
'# 26-Feb-2021  m15t3r_gr1mm    change nomenclature to "updater" to avoid confusion with official launcher
'# 10-Jan-2021  m15t3r_gr1mm    update to notes - taskbar bug workaround (Jan Ainnir Mayen)
'# 02-Jan-2021  m15t3r_gr1mm    initial delivery
'#
'#  ** This is a free an open source program - not to be used for profit **
'###############################################################################

Sub Main()

   lcReVolt = "rvgl.exe"
   lbNoUpdate = false

   ' parse each argument passed in via .lnk to the main RVGL binary
   Dim lnNumArgs : lnNumArgs = Wscript.Arguments.Count
   For lnIx = 0 to (lnNumArgs-1) Step 1
      ' check to see if user wants to bypass automatic updates
      if StrComp(Wscript.Arguments(lnIx),"-noupdate") = 0 then
         lbNoUpdate = true
      else
         lcReVolt = lcReVolt & " " & Wscript.Arguments(lnIx)
      end if
   Next

   ' lcDq translates to just one literal double quote at the lowest level
   ' (i.e. - PS subshell invoked from vbs, note that normal PS escapes still apply)
   lcDq = chr(34)&chr(34)&chr(34)

   ' powershell assemblies to load
   lcAssemblies = "<# Required for VBS download calls #>" &_
      "Add-Type -AssemblyName Microsoft.VisualBasic;" &_
      "<# Modal dialog boxes #>" &_
      "Add-Type -AssemblyName PresentationFramework;" &_
      "<# Class needed for interacting with tarball #>" &_
      "Add-Type -AssemblyName System.IO.Compression.FileSystem;" &_
      "<# ComboBox and form selection #>" &_
      "Add-Type -AssemblyName System.Windows.Forms;" &_
      "Add-Type -AssemblyName System.Drawing;"

   ' function for displaying drop-down with rvgl binary versions
   ' source: https://gist.github.com/sdovnic/838a778a2ecd99bd3dea98930d7558af
   lcComboBox = "function Show-ComboBox {" &_
      "    [CmdLetBinding()]" &_
      "    Param (" &_
      "        [Parameter(Mandatory=$true)] $Items," &_
      "        [Parameter(Mandatory=$false)] [switch] $ReturnIndex," &_
      "        [Parameter(Mandatory=$true)] [string] $FormTitle," &_
      "        [Parameter(Mandatory=$false)] [string] $ButtonText = "&lcDq&"OK"&lcDq &_
      "    )" &_
      "    process {" &_
      "        $ComboBoxSize = New-Object System.Drawing.Size;" &_
      "        $ComboBoxSize.Height = 20;" &_
      "        $ComboBoxSize.Width = 150;" &_
      "        $ComboBoxPosition = New-Object -TypeName System.Drawing.Point;" &_
      "        $ComboBoxPosition.X = 10;" &_
      "        $ComboBoxPosition.Y = 10;" &_
      "        $ComboBox = New-Object -TypeName System.Windows.Forms.ComboBox;" &_
      "        $ComboBox.Location = $ComboBoxPosition;" &_
      "        $ComboBox.DataBindings.DefaultDataSourceUpdateMode = 0;" &_
      "        $ComboBox.FormattingEnabled = $true;" &_
      "        $ComboBox.Name = "&lcDq&"comboBox1"&lcDq&";" &_
      "        $ComboBox.TabIndex = 0;" &_
      "        $ComboBox.Size = $ComboBoxSize;" &_
      "        $Items | foreach {" &_
      "            $ComboBox.Items.Add($_);" &_
      "            $ComboBox.SelectedIndex = 0;" &_
      "        } | Out-Null;" &_
      "        $ComboBox.SelectedIndex = 4;" &_
      "        $ButtonSize = New-Object -TypeName System.Drawing.Size;" &_
      "        $ButtonSize.Height = 23;" &_
      "        $ButtonSize.Width = 60;" &_
      "        $ButtonPosition = New-Object -TypeName System.Drawing.Point;" &_
      "        $ButtonPosition.X = 175;" &_
      "        $ButtonPosition.Y = 9;" &_
      "        $ButtonOnClick = {" &_
      "            $global:SelectedItem = $ComboBox.SelectedItem;" &_
      "            $global:SelectedIndex = $ComboBox.SelectedIndex;" &_
      "            $Form.Close();" &_
      "        };" &_
      "        $Button = New-Object -TypeName System.Windows.Forms.Button;" &_
      "        $Button.TabIndex = 2;" &_
      "        $Button.Size = $ButtonSize;" &_
      "        $Button.Name = "&lcDq&"button1"&lcDq&";" &_
      "        $Button.UseVisualStyleBackColor = $true;" &_
      "        $Button.Text = $ButtonText;" &_
      "        $Button.Location = $ButtonPosition;" &_
      "        $Button.DataBindings.DefaultDataSourceUpdateMode = 0;" &_
      "        $Button.add_Click($ButtonOnClick);" &_
      "        $FormSize = New-Object -TypeName System.Drawing.Size;" &_
      "        $FormSize.Height = 50;" &_
      "        $FormSize.Width = 250;" &_
      "        $Form = New-Object -TypeName System.Windows.Forms.Form;" &_
      "        $Form.AutoScaleMode = 0;" &_
      "        $Form.Text = $FormTitle;" &_
      "        $Form.Name = "&lcDq&"form1"&lcDq&";" &_
      "        $Form.DataBindings.DefaultDataSourceUpdateMode = 0;" &_
      "        $Form.ClientSize = $FormSize;" &_
      "        $Form.FormBorderStyle = 1;" &_
      "        $Form.Controls.Add($Button);" &_
      "        $Form.Controls.Add($ComboBox);" &_
      "        $Form.Icon = [System.Drawing.Icon]::ExtractAssociatedIcon("&lcDq&"$pwd\rvgl.exe"&lcDq&");" &_
      "        $Form.ShowDialog() | Out-Null;" &_
      "    }" &_
      "    end {" &_
      "        $SelectedItem = $global:SelectedItem;" &_
      "        $SelectedIndex = $global:SelectedIndex;" &_
      "        Clear-Variable -Name "&lcDq&"SelectedItem"&lcDq&" -Force -Scope global;" &_
      "        if ($ReturnIndex) {" &_
      "            return $SelectedIndex;" &_
      "        } else {" &_
      "            return $SelectedItem;" &_
      "        }" &_
      "    }" &_
      "}"


   ' function for downloading file with progress bar
   ' (faster than invoke-webrequest since the progress is updated less frequently)
   lcGetFileVb = "function Get-FileVB {" &_
      "   param(" &_
      "       [Parameter(Mandatory=$true)]" &_
      "       $url, " &_
      "       $destinationFolder="&lcDq&"$env:USERPROFILE\Downloads"&lcDq&"," &_
      "       [switch]$includeStats" &_
      "   )" &_
      "   <# resolve potential redirect #>" &_
      "   $response = [System.Net.WebRequest]::Create($url).GetResponse();" &_
      "   $url = $response.ResponseUri.OriginalString;" &_
      "   $response.Close();" &_
      "   $destination = Join-Path $destinationFolder ($url | Split-Path -Leaf);" &_
      "   $net = New-Object Microsoft.VisualBasic.Devices.Network;" &_
      "   $start = Get-Date;" &_
      "   <# signature DownloadFile(url, destination, username, password, [bool]showUI, [int]connecdtionTimeOutInMS,[Microsoft.VisualBasic.FileIO.UICancelOption]OnUserCancel) #>" &_
      "   $net.DownloadFile($url, $destination, '', '', $true, 500, [Microsoft.VisualBasic.FileIO.UICancelOption]::DoNothing );" &_
      "   $elapsed = ((Get-Date) - $start).ToString('hh\:mm\:ss');" &_
      "   $totalSize = (Get-Item $destination).Length | Get-FileSize;" &_
      "   if ($includeStats.IsPresent) {" &_
      "       [PSCustomObject]@{Name=$MyInvocation.MyCommand;TotalSize=$totalSize;Time=$elapsed};" &_
      "   }" &_
      "   Get-Item $destination | Unblock-File;" &_
      "}"


   ' this is the main powershell code block - to be passed in as a script block to a subshell
   lcMain = "$install_exists = Test-Path -Path "&lcDq&"$pwd\rvgl_changelog.txt"&lcDq&";" &_
      "<# Only proceed if install exists" &_
      " # (it should since this script lives within the install) #>" &_
      "if ($install_exists -eq $true)" &_
      "{" &_
      "   $current_ver = Get-Content rvgl_changelog.txt | Where-Object { $_ -like "&lcDq&"*:"&lcDq&" } | select -Index 0;" &_
      "   $current_ver = $current_ver.Replace("&lcDq&":"&lcDq&","&lcDq&lcDq&");" &_
      "   $regex = '[a-zA-Z]';" &_
      "   <# remove alphanumeric (apparently that part isn't stored in rvgl_version.txt) #>" &_
      "   $current_ver = $current_ver -replace $regex,'';" &_
      "   $release_page = wget -uri "&lcDq&"https://distribute.re-volt.io/releases/rvgl_version.txt"&lcDq&";" &_
      "   $update_ver = $release_page.Content.Trim();" &_
      "   <# if version is a mismatch, prompt the user #>" &_
      "   if ($current_ver -ne $update_ver)" &_
      "   {" &_
      "      $statusMsg =  [System.Windows.MessageBox]::Show("&lcDq&"RVGL version $update_ver is available`nDownload now?"&lcDq&",'Update Required','YesNo','Warning');" &_
      "      switch  ($statusMsg) {" &_
      "         'Yes' {" &_
      "            <# Download the tarball" &_
      "             # Robust location to the users downloads folder #>" &_
      "            $downloads = (New-Object -ComObject Shell.Application).NameSpace('shell:Downloads').Self.Path;" &_
      "            $Selection = Show-ComboBox -Items ("&lcDq &_
      "rvgl_full_win32_basic"&lcDq&","&lcDq &_
      "rvgl_full_win32_online"&lcDq&","&lcDq &_
      "rvgl_full_win32_original"&lcDq&","&lcDq &_
      "rvgl_full_win64_basic"&lcDq&","&lcDq &_
      "rvgl_full_win64_online"&lcDq&","&lcDq &_
      "rvgl_full_win64_original"&lcDq &_
      ") -FormTitle "&lcDq&"RVGL version"&lcDq&" -ButtonText "&lcDq&"OK"&lcDq&";" &_
      "            $Selection = $Selection + "&lcDq&".zip"&lcDq&";" &_
      "            $tarball = "&lcDq&"$downloads\$Selection"&lcDq&";" &_
      "            Get-FileVB "&lcDq&"https://distribute.re-volt.io/releases/$Selection"&lcDq&" -includeStats;" &_
      "            <# Expand-Archive is faster, but this is the fastest method available with a gui progress bar #>" &_
      "            $shell = New-Object -ComObject Shell.Application;" &_
      "            $shell.NameSpace($pwd.Path).copyhere($shell.NameSpace($tarball).Items(),16);" &_
      "         }" &_
      "         'No' {" &_
      "            <# Do Nothing #>" &_
      "         }" &_
      "      }" &_
      "   }" &_
      "}"

   lcCodeblock =  "powershell.exe Invoke-Command -ScriptBlock { " & lcAssemblies & lcComboBox & lcGetFileVb & lcMain & " }"

   'MsgBox(lcCodeblock)
   Dim objShell
   Set objShell = Wscript.CreateObject("Wscript.Shell")

   ' check for any updates if user has not bypassed
   if lbNoUpdate = false then
      objShell.Run(lcCodeblock), 0, true
   end if

   ' always start the game
   objShell.Run(lcReVolt)

   Set objShell = Nothing

End Sub

Call Main()